# Python script to plot data from snap****.out using the script by Calvin Lau
# The code is based in the orginal 'snapshot_v2'
# This code plot the pdf of the pitch, energy and angles


import math
import numpy
import matplotlib.pyplot as plt
import scipy
import matplotlib.gridspec as gridspec
def readsnapshot(fname,kind=0): 
    """
    This reads snap.out files into something that can be read by python more easily.
    kind=0 reads all data
    kind=1 reads just profile
    kind=2 reads just pdf
    kind=3 reads just poloidal
    kind=4 reads just flux
    kind=5 reads just the 2DPDF for pitch and angle

	This function was written by Calvin Lau
    """
  #try:  
    lines = open(fname, 'r').readlines()
    nspecies=int(lines[0])  # of species: ion, electron, EP, impuries
    nfield=int(lines[1])    # of field variables: phi, a_para, fluidne
    nvgrid=int(lines[2])    # of grids in energy and pitch
    mpsi=int(lines[3])      # of radial grids: mpsi+1
    mtgrid=int(lines[4])    # of poloidal grids
    mtoroidal=int(lines[5]) # of toroidal (parallel) grids
    tmax=float(lines[6])    #upper bound of temperature
    #print "nspecies=",nspecies,",","nfield=",nfield,",","nvgrid=",nvgrid,",","mpsi=",mpsi,",","mtgrid=",mtgrid,",","mtoroidal=",mtoroidal,",","tmax=",tmax

    sect1=mpsi*6*nspecies
    sect2=nvgrid*4*nspecies
    sect3=mtgrid*mpsi*(nfield+2)
    sect4=mtgrid*mtoroidal*nfield
    sect5=nvgrid*nvgrid*2*nspecies

    profileseries=numpy.zeros(sect1,dtype=float)
    pdfseries=numpy.zeros(sect2,dtype=float)
    poloidataseries=numpy.zeros(sect3,dtype=float)
    fluxdataseries=numpy.zeros(sect4,dtype=float)
    pdf2dmuenergyseries=numpy.zeros(sect5,dtype=float)
    
    profile=numpy.zeros((mpsi,6,nspecies),dtype=float)
    pdf=numpy.zeros((nvgrid,4,nspecies),dtype=float)
    poloidata=numpy.zeros((mtgrid,mpsi,nfield+2),dtype=float)
    fluxdata=numpy.zeros((mtgrid,mtoroidal,nfield),dtype=float)
    pdf2dmuenergy=numpy.zeros((nvgrid,nvgrid,2,nspecies),dtype=float)

    ## this separates the files into the four sections as a series of float numbers
    if kind==0 or kind==1:
        for i in range(0,sect1):
            profileseries[i]=float(lines[7+i])
    if kind==0 or kind==2:
        for i in range(0,sect2): 
            pdfseries[i]=float(lines[7+sect1+i])
    if kind==0 or kind==3:
        for i in range(0,sect3):
            poloidataseries[i]=float(lines[7+sect1+sect2+i])
    if kind==0 or kind==4:
        for i in range(0,sect4): 
            fluxdataseries[i]=float(lines[7+sect1+sect2+sect3+i])
    #the 2D PDF for pitch (lambda) and energy 
    if  kind==5:
        for i in range(0,sect5): 
            pdf2dmuenergyseries[i]=float(lines[7+sect1+sect2+sect3+sect4+i])
        
    ## this reorganizes the four sections into the data format we expect in idl
    if kind==0 or kind==1:
        for i in range(0,nspecies):
            for j in range(0,6):
                for k in range(0,mpsi):
                    profile[k,j,i]=profileseries[k+(j*mpsi)+(i*mpsi*6)]
                    
    if kind==0 or kind==2:
        for i in range(0,nspecies):
            for j in range(0,4):
                for k in range(0,nvgrid):
                    pdf[k,j,i]=pdfseries[k+(j*nvgrid)+(i*nvgrid*4)]
                    
    if kind==0 or kind==3:            
        for i in range(0,nfield+2):
            for j in range(0,mpsi):
                for k in range(0,mtgrid):
                    poloidata[k,j,i]=poloidataseries[k+(j*mtgrid)+(i*mpsi*mtgrid)]
    
    if kind==0 or kind==4:    
        for i in range(0,nfield):
            for j in range(0,mtoroidal):
                for k in range(0,mtgrid):
                    fluxdata[k,j,i]=fluxdataseries[k+(j*mtgrid)+(i*mtgrid*mtoroidal)]  
  
    if kind==5:     
        for i in range(0,nspecies):
            for j in range(0,2):
               for k in range(0,nvgrid):
                  for l in range(0,nvgrid):
                    pdf2dmuenergy[l,k,j,i]=pdf2dmuenergyseries[l+(k*nvgrid)+(j*nvgrid*nvgrid)+(i*nvgrid*nvgrid*2)]   

    if kind==0:
        return (profile, pdf, poloidata,fluxdata,nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax)
    elif kind==1:
        return (profile,nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax)
    elif kind==2:
        return (pdf,nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax)
    elif kind==3:
        return (poloidata,nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax)
    elif kind==4:
        return (fluxdata,nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax)
    elif kind==5:
        return (pdf2dmuenergy,nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax)


#########################################################################################################
## radial plot
###########################
def radialprof(f,mpsi,mtgrid):
    """
    Based on the original IDL code
    """
    x=range(0,mpsi)
    y1=numpy.zeros(mpsi)
    y2=numpy.zeros(mpsi)
    
    for i in range(0,mpsi-1):
        y1[i]=f[i,0]	#the zero can be modified
        y2[i]=f[i,1]	#the zero can be modified


    #fig = plt.figure(figsize=(10, 10)) #necessary to get a .png file?? 
    fig, ax = plt.subplots(figsize=(8, 8))
    
    #MULTIPLE PLOTS

    ax = plt.subplot(2,1,1)

    ymin1=min(y1)
    ymax1=max(y1)
    xmin=min(x)
    xmax=max(x)
    ax.plot(x,y1,color='g',linestyle="-" ,marker='p',label='None', linewidth=5.5) 

    ax.set_ylim(ymin1,ymax1)
    ax.set_xlim(xmin,xmax)
    ax.set_title(r'Density  (Full f)',fontsize=20)

    ax.tick_params(axis='x', labelsize=20)	#size axis values
    ax.tick_params(axis='y', labelsize=20)


    #Second plot
    ax2 = plt.subplot(2,1,2)

    ymin2=min(y2)
    ymax2=max(y2)
    #xmin=min(x)
    #xmax=max(x)
    ax2.plot(x,y2,color='g',linestyle="-" ,marker='p',label='None', linewidth=5.5) 

    ax2.set_ylim(ymin2,ymax2)
    ax2.set_xlim(xmin,xmax)

    ax2.set_title(r'Density  ($\delta f$)',fontsize=20)
    ax2.set_xlabel(r'Radius',fontsize=20)

    ax2.tick_params(axis='x', labelsize=20)	#size axis values
    ax2.tick_params(axis='y', labelsize=20)

    plt.tight_layout()
    #plt.show()
    #plt.clf()
    return fig


#########################################################################################################
## poloidal plot phi
####################
def poloidal(x,y,f,r0):
   
    #length unit in GTC are in terms of R0
    #to plot real units
    x=x*r0 
    y=y*r0   
 
    zmax=abs(f).max()
    zmin=-abs(f).max()
    xmax=x.max()
    xmin=x.min()
    ymax=y.max()
    ymin=y.min()
  
    #print zmax, zmin

    filtermax=1#0.85    #to better visualize the intermediate values (1 by default)
    zmax=filtermax#*abs(f).max()
    zmin=-filtermax#*abs(f).max()
    #print 'Filter in the colormap to show lower values'

    fig, ax = plt.subplots(figsize=(8, 8))
    #p1=plt.contourf(x,y,f,clevel,cmap='jet')
    p1=plt.pcolormesh(x, y, f/abs(f).max(), cmap='seismic',vmin=zmin, vmax=zmax)


    plt.xlim([xmin,xmax])
    plt.ylim([ymin,ymax])

    # COLORMAP
    cbar = plt.colorbar(pad = 0.02)
    
    ax.set_aspect('equal')
    
    plt.tight_layout()
    #plt.show()
    #plt.clf()
    return fig

#########################################################################################################
## flux surface 
####################
def fluxsurface(f,mtoroidal,mtgrid,toroidaln=1):

    #x=range(0,mtoroidal)
    #y=range(0,mtgrid)
    x=[]
    y=[]


    for i in xrange(mtgrid):
	y.append(2.0*3.141592*float(i)/float(mtgrid)-3.14159)
    for i in xrange(mtoroidal):
        x.append(2.0*3.141592*float(i)/float(mtoroidal*toroidaln))

    filtermax=0.7#0.85    #to better visualize the intermediate values (1 by default)
    zmax=filtermax#*abs(f).max()
    zmin=-filtermax#*-abs(f).max()
    xmax=max(x)
    xmin=min(x)
    ymax=max(y)
    ymin=min(y)
  
    #print zmax, zmin

    fig, ax = plt.subplots(figsize=(8, 8))
    #p1=plt.contourf(x,y,f,clevel,cmap='jet')
    p1=plt.pcolormesh(x, y, f/abs(f).max(), cmap='coolwarm',vmin=zmin, vmax=zmax)


    plt.xlim([xmin,xmax])
    plt.ylim([ymin,ymax])

    # COLORMAP
    cbar = plt.colorbar(pad = 0.02)

    ax.set_xlabel(r'Parallel direction',fontsize=32)
    ax.set_ylabel(r'Poloidal direction',fontsize=32)
    ax.tick_params(axis='x', labelsize=25)	#size axis values
    ax.tick_params(axis='y', labelsize=25)

    cbar.ax.tick_params(labelsize=25)

    # to set the exact position and format of ticks
    ax.yaxis.set_ticks(numpy.arange(ymin,ymax*1.001,3.14159/2))
    #ax.xaxis.set_major_formatter(FormatStrFormatter('%1.0f'))

    ax.xaxis.set_ticks(numpy.arange(xmin,xmax*1.101,xmax/2))

    #custom ticklabel
    fig.canvas.draw()
    labels = [item.get_text() for item in ax.get_yticklabels()]
    labels=['$-\pi$','$-\pi/2$','$0$','$\pi/2$','$\pi$']
    ax.set_yticklabels(labels)

    labels = [item.get_text() for item in ax.get_yticklabels()]
    labels=['$0$','$2\pi/$'+str(2*toroidaln),'$2\pi/$'+str(toroidaln)]
    ax.set_xticklabels(labels)

    #ax.set_aspect('equal')
    
    plt.tight_layout()
    #plt.show()
    #plt.clf()
    return fig

#########################################################################################################
## 2D pdf surface 
####################
def plt2Dpdf(f,xgrid,ygrid,plotkind,toroidaln,tmax):

    #x=range(0,mtoroidal)
    #y=range(0,mtgrid)
    x=[]
    y=[]

    if plotkind==5:
      for i in xrange(ygrid):
         y.append((1/0.8)*float(i)/float(ygrid))
         #y.append(tmax*float(i)/float(ygrid))
      for i in xrange(xgrid):
          x.append(tmax*float(i)/float(xgrid))
      x_label='$E (T_e)$'
      y_label=r'$\mu B_{\mathrm{max}}/E$'
      #y_label=r'$\mu B_{0}$'

#    for i in xrange(nvgrid):
#	    y.append(-1+2*float(i)/float(nvgrid))
	    #y.append(2.0*3.141592*float(i)/float(nvgrid))
    #if(toroidaln==1):
#    for i in xrange(numthetabins):
#        x.append(2.0*3.141592*float(i)/float(numthetabins))
        #x.append(2.0*3.141592*float(i)/float(numthetabins*toroidaln))
        #x.append(float(i)/float(numthetabins*toroidaln))

    filtermax=0.85    #to better visualize the intermediate values (1 by default)
    zmax=filtermax*abs(f).max()
    zmin=filtermax*-abs(f).max()
    xmax=max(x)
    xmin=min(x)
    ymax=max(y)
    ymin=min(y)
  
    #print zmax, zmin

    fig, ax = plt.subplots(figsize=(8, 8))
    #p1=plt.contourf(x,y,f,clevel,cmap='jet')
    p1=plt.pcolormesh(x, y, f, cmap='jet')#,vmin=zmin, vmax=zmax)

    if plotkind==5:
       yaux=numpy.array(x)
       #yaux=(1/1.2)*yaux
       yaux=0*yaux+(0.725) #horizontal line
       ax.plot(x,yaux,color='w',linestyle="--" ,linewidth=3.5)
       ax.annotate(r'$\mu B_\mathrm{min}/E$',xy=(3.5,0.7),size=22,color='w')#, backgroundcolor='w')

    plt.xlim([xmin,xmax])
    #plt.ylim([ymin,ymax])
    plt.ylim(0.5,1.25)

    # COLORMAP
    cbar = plt.colorbar(pad = 0.02)

#    ax.set_xlabel(r'Toroidal direction',fontsize=32)
    ax.set_xlabel(x_label,fontsize=32)
    ax.set_ylabel(y_label,fontsize=32)
#    ax.set_ylabel(r'Poloidal angle',fontsize=32)
    ax.tick_params(axis='x', labelsize=25)	#size axis values
    ax.tick_params(axis='y', labelsize=25)

    cbar.ax.tick_params(labelsize=25)

    #ax.set_aspect('equal')

    plt.tight_layout()
    #plt.show()
    #plt.clf()
    return fig


#*************
#************************
#**********************************
#         MAIN PROGRAM
#**********************************
#************************
#*************

#import snapshot_modules

print " "

snapshotnumber='0001500'

toroidaln=5	#1 for full torus,  otherwise 2*pi/toroidaln

#Set the type of plot you need
# plotkind=1--> Plot the radial profile
# plotkind=3--> Plot the phi poloidal cross section
# plotkind=4 --> Plot the phi flux surface
# plotkind=5 --> Plot the 2D PDF for mu (lambda) and energy 

plotkind=5

specie=1 #0->ion, 1->electron

printplot=1  #Print the plot in file? 1--> yes

#major radius
r0=5.6

#**********************
#**********************
filename='snap'+snapshotnumber+'.out'

print ' '
print '   ****************************'
print '            PLOT SNAPSHOT'
print '   ****************************'
print ' '
print ' '
print ' Input file:', filename
print ' '

if specie==0:
   stringsp='ion'
if specie==1:
   stringsp='electron'

if plotkind == 1:
	#TO DO: specify which profile you want
  #Load the profiles
  print ' Plotting a radial profile...'

  outputfile='radial_profile_'+snapshotnumber

  profile,nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax =readsnapshot(filename,plotkind)

  #Print the radial profile 
  fig= radialprof(profile, mpsi, mtgrid)


if plotkind == 3:
  #*****
  #load the poloidal cross sections
  print ' Plotting phi in a poloidal cross section...'

  outputfile='phi_pol_cross_section_'+snapshotnumber

  #poloidata,nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax = snapshot_modules.readsnapshot(filename,3)

  poloidata,nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax = readsnapshot(filename,plotkind)

  #Print a poloidal plane
  # poloidal grid position in polar coordinates
  x=poloidata[:,:,nfield]
  y=poloidata[:,:,nfield+1]
  #phi value
  f=poloidata[:,:,0]

  #fig = snapshot_modules.poloidal(x,y,f)
  fig = poloidal(x,y,f,r0)


if plotkind == 4:
  #*******
  #Load the flux surface
  print ' Plotting phi in the flux surface...'
  
  outputfile='phi_flux_surface_'+snapshotnumber
   
  fluxdata,nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax = readsnapshot(filename,plotkind)

  phifluxsurface=fluxdata[:,:,0]

  #shift the values in pol direction to center it in pol=0
  phishift=numpy.zeros((mtgrid,mtoroidal),dtype=float)

  for j in range(0,mtgrid):
    joffset=j+mtgrid/2
    if j>=mtoroidal/2:
       joffset=j-mtgrid/2-1
    phishift[j,:]=phifluxsurface[joffset,:]         

  fig = fluxsurface(phishift,mtoroidal,mtgrid,toroidaln)

if plotkind == 5:

  #*******
  #Load the 2D PDF
  print ' Plotting 2DPDF for mu and energy...'
  
  outputfile='2D_PDF_mu_energy_'+stringsp+snapshotnumber
   
  pdf2d,nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax = readsnapshot(filename,plotkind)

  pdf2ds=pdf2d[:,:,1,specie]

  fig = plt2Dpdf(pdf2ds,nvgrid,nvgrid,plotkind,1,tmax)
 #we have set toroidaln=1 because we dont care about toroidal here

if printplot == 1:
  #*************************
  # Save picture
  #*************************
  
  print ' '
  print ' Printing the plot'
  #fig.set_size_inches(13,9)
  #fig.get_dpi()
  #fig.set_dpi(600)
  fig.savefig(outputfile+'.png',bbox_inches='tight')

  ############################333

plt.show()

print " "


